#include "PokktCocosManager.h"

// pokkt event definitions
const std::string PokktCocosManager::VideoClosedEvent = "videoClosed";
const std::string PokktCocosManager::VideoDisplayedEvent = "videoDisplayed";
const std::string PokktCocosManager::VideoSkippedEvent = "videoSkipped";
const std::string PokktCocosManager::VideoCompletedEvent = "videoCompleted";
const std::string PokktCocosManager::VideoGratifiedEvent = "videoGratified";
const std::string PokktCocosManager::DownloadCompletedEvent = "downloadCompleted";
const std::string PokktCocosManager::DownloadFailedEvent = "downloadFailed";
const std::string PokktCocosManager::CoinResponseEvent = "coinResponse";
const std::string PokktCocosManager::CoinResponseWithTransIdEvent = "coinResponseWithTrId";
const std::string PokktCocosManager::CoinResponseFailedEvent = "coinResponseFailed";
const std::string PokktCocosManager::CampaignAvailabilityEvent = "campaignAvailability";
const std::string PokktCocosManager::OfferwallClosedEvent = "offerwallClosedEvent";

bool PokktCocosManager::_isDebugEnabled = true;
std::map<std::string, PokktCocosManager::ListenerTargetPair*> PokktCocosManager::_eventMap;

void PokktCocosManager::handleNativeMessage(std::string operation, std::string param)
{
    POKKTLOG("[POKKT-CPP] message received! operation: %s, param: %s", operation.c_str(), param.c_str());
    
    ListenerTargetPair *l = _eventMap[operation];
    if (l != NULL)
    {
        EVT_SELECTOR listener = l->listener;
        cocos2d::Node *target = l->target;
        (target->*listener)(param);
    }
}

void PokktCocosManager::setListener(std::string eventName, EVT_SELECTOR listener, cocos2d::Node *target)
{
    ListenerTargetPair *pair = new ListenerTargetPair();
    pair->listener = listener;
    pair->target = target;
    _eventMap[eventName] = pair;
}

void PokktCocosManager::removeListener(std::string eventName)
{
    CC_SAFE_DELETE(_eventMap[eventName]);
    _eventMap[eventName] = NULL;
}

static std::string getParamString(PokktCocosConfig *pokktCocosConfig)
{
    std::string integrationType = "";
    switch (pokktCocosConfig->getIntegrationType()) {
        case PokktCocosConfig::INTEGRATION_TYPE_ALL:
            integrationType = "0";
            break;
        case PokktCocosConfig::INTEGRATION_TYPE_OFFERWALL:
            integrationType = "1";
            break;
        case PokktCocosConfig::INTEGRATION_TYPE_VIDEO:
            integrationType = "2";
            break;
        default:
            integrationType = "0";
            break;
    }
    std::string paramString =
    pokktCocosConfig->getSecurityKey() + "," + pokktCocosConfig->getApplicationId() + "," + integrationType + "," + (pokktCocosConfig->getAutoCacheVideo() ? "true" : "false");
    
    return paramString;
}

void PokktCocosManager::setParams(PokktCocosConfig *pokktCocosConfig)
{
    PokktNativeExtension::performOperation(PokktNativeExtension::SetParams, getParamString(pokktCocosConfig));
    
    PokktNativeExtension::performOperation(PokktNativeExtension::SetDebug, _isDebugEnabled ? "true" : "false");
}

// common methods

void PokktCocosManager::initPokkt(PokktCocosConfig *pokktCocosConfig)
{
    setParams(pokktCocosConfig);
}

void PokktCocosManager::setDebug(bool value)
{
    _isDebugEnabled = value;
    
    PokktNativeExtension::performOperation(PokktNativeExtension::SetDebug, _isDebugEnabled ? "true" : "false");
}

bool PokktCocosManager::getDebug()
{
    return _isDebugEnabled;
}

void PokktCocosManager::showLog(std::string message)
{
    PokktNativeExtension::performOperation(PokktNativeExtension::ShowLog, message);
}

void PokktCocosManager::showToast(std::string message)
{
    PokktNativeExtension::performOperation(PokktNativeExtension::ShowToast, message);
}

// session methods

void PokktCocosManager::startSession(PokktCocosConfig *pokktCocosConfig)
{
    setParams(pokktCocosConfig);
    PokktNativeExtension::performOperation(PokktNativeExtension::StartSession);
}
void PokktCocosManager::endSession()
{
    PokktNativeExtension::performOperation(PokktNativeExtension::EndSession);
}

// OfferWall methods

void PokktCocosManager::getCoins(PokktCocosConfig *pokktCocosConfig)
{
    PokktNativeExtension::performOperation(PokktNativeExtension::GetCoins, pokktCocosConfig->getOfferWallAssetValue());
}

void PokktCocosManager::getPendingCoins()
{
    PokktNativeExtension::performOperation(PokktNativeExtension::GetPendingCoins);
}

void PokktCocosManager::checkOfferWallCampaign()
{
    PokktNativeExtension::performOperation(PokktNativeExtension::CheckOfferWallCampaign);
}

// Video Methods

void PokktCocosManager::getVideo(PokktCocosConfig *pokktCocosConfig)
{
    pokktCocosConfig->setIncentivised(true);
    PokktNativeExtension::performOperation(PokktNativeExtension::GetVideo, pokktCocosConfig->getScreenName());
}

void PokktCocosManager::getVideoNonIncent(PokktCocosConfig *pokktCocosConfig)
{
    pokktCocosConfig->setIncentivised(false);
    PokktNativeExtension::performOperation(PokktNativeExtension::GetVideoNonIncent, pokktCocosConfig->getScreenName());
}

void PokktCocosManager::cacheVideoCampaign()
{
    PokktNativeExtension::performOperation(PokktNativeExtension::CacheVideoCampaign);
}

bool PokktCocosManager::isVideoAvailable()
{
    return PokktNativeExtension::isVideoAvailable();
}

double PokktCocosManager::getVideoVc()
{
    return PokktNativeExtension::getVideoVc();
}

// optional methods

std::string PokktCocosManager::getVersion()
{
    return PokktNativeExtension::getPokktSDKVersion();
}